@@ -187,7 +187,7 @@ class SimpleEditor ( Editor ):
         item_trait    = trait_handler.item_trait
         factory       = self.factory
         list_sizer    = wx.FlexGridSizer(
-                            0, (1 + resizable) * factory.columns, 0, 0 )
+                            len(self.value), (1 + resizable) * factory.columns, 0, 0 )
         j = resizable
         for i in range( factory.columns ):
             list_sizer.AddGrowableCol( j )
@@ -197,10 +197,11 @@ class SimpleEditor ( Editor ):
         index         = 0
         width, height = 0, 0
 
-        is_fake = (resizable and (len( values ) == 0))
+        is_fake = (resizable and (values is None or len( values ) == 0))
         if is_fake:
             values = [ item_trait.default_value()[1] ]
 
+        panel_height = 0
         editor = self._editor
         for value in values:
             width1 = height = 0
@@ -231,7 +232,8 @@ class SimpleEditor ( Editor ):
             pcontrol.SetMinSize( size )
             width  = max( width, width1 + width2 )
             height = max( height, height2 )
-
+            panel_height += height
+            
             if resizable:
                 list_sizer.Add( control, 0, wx.LEFT | wx.RIGHT, 2 )
 
@@ -240,6 +242,10 @@ class SimpleEditor ( Editor ):
 
         list_pane.SetSizer( list_sizer )
 
+        if not self.mutable:
+            #list_sizer.SetDimension(0,0,width, panel_height)
+            list_pane.SetInitialSize(list_sizer.GetSize())
+                    
         if is_fake:
             self._cur_control = control
             self.empty_list()
@@ -255,12 +261,13 @@ class SimpleEditor ( Editor ):
         if width == 0:
             width = 100
             
-        if height == 0:
-            height = 20
+        if panel_height == 0:
+            panel_height = 20
 
         list_pane.SetMinSize( wx.Size(
              width + ((trait_handler.maxlen > rows) * scrollbar_dx),
-             height * rows ) )
+             panel_height) )
+        
         list_pane.SetupScrolling()
         list_pane.GetParent().Layout()
 
@@ -339,8 +346,9 @@ class SimpleEditor ( Editor ):
         """ Displays the empty list editor popup menu.
         """
         self._cur_control = control
-        control.PopupMenuXY( MakeMenu( self.empty_list_menu, self, True,
-                                       control ).menu, 0, 0 )
+        menu = MakeMenu( self.empty_list_menu, self, True, self.control ).menu
+        self.control.PopupMenu( menu, control.GetPosition() )
+        menu.Destroy()
 
     #---------------------------------------------------------------------------
     #  Displays the list editor popup menu:
@@ -354,7 +362,7 @@ class SimpleEditor ( Editor ):
         control.SetFocus()
         proxy    = control.proxy
         index    = proxy.index
-        menu     = MakeMenu( self.list_menu, self, True, control ).menu
+        menu     = MakeMenu( self.list_menu, self, True, self.control ).menu
         len_list = len( proxy.list )
         not_full = (len_list < self._trait_handler.maxlen)
         self._menu_before.enabled( not_full )
@@ -364,7 +372,8 @@ class SimpleEditor ( Editor ):
         self._menu_top.enabled( index > 0 )
         self._menu_down.enabled(   index < (len_list - 1) )
         self._menu_bottom.enabled( index < (len_list - 1) )
-        control.PopupMenuXY( menu, 0, 0 )
+        self.control.PopupMenu( menu, control.GetPosition() )
+        menu.Destroy()
 
     #---------------------------------------------------------------------------
     #  Adds a new value at the specified list index:
@@ -872,4 +881,4 @@ class DockableListElement ( DockableViewElement ):
                 self.editor.selected = value[1]
                 break
 
-### EOF #######################################################################
\ No newline at end of file
+### EOF #######################################################################
