@@ -49,11 +49,36 @@ from constants \
 from helper \
     import TraitsUIPanel, Slider
 
+#-------------------------------------------------------------------------------
+#  'BaseRangeEditor' class:
+#-------------------------------------------------------------------------------
+
+class BaseRangeEditor ( Editor ):
+    """ The base class for Range editors. Using an evaluate trait, if specified,
+        when assigning numbers the object trait.
+    """
+    
+    #---------------------------------------------------------------------------
+    #  Trait definitions:
+    #---------------------------------------------------------------------------
+
+    # Function to evaluate floats/ints
+    evaluate = Any
+
+    #---------------------------------------------------------------------------
+    #  Sets the associated object trait's value:
+    #---------------------------------------------------------------------------
+    
+    def _set_value ( self, value ):
+        if self.evaluate is not None:
+            value = self.evaluate( value )
+        Editor._set_value( self, value )
+
 #-------------------------------------------------------------------------------
 #  'SimpleSliderEditor' class:
 #-------------------------------------------------------------------------------
 
-class SimpleSliderEditor ( Editor ):
+class SimpleSliderEditor ( BaseRangeEditor ):
     """ Simple style of range editor that displays a slider and a text field.
 
     The user can set a value either by moving the slider or by typing a value
@@ -72,10 +97,7 @@ class SimpleSliderEditor ( Editor ):
 
     # Formatting string used to format value and labels
     format = Str
-
-    # Function to evaluate textual user input
-    evaluate = Any
-
+    
     # Flag indicating that the UI is in the process of being updated
     ui_changing = Bool( False )
 
@@ -106,18 +128,18 @@ class SimpleSliderEditor ( Editor ):
 
         self.sync_value( factory.low_name,  'low',  'from' )
         self.sync_value( factory.high_name, 'high', 'from' )
-        #low  = self.low
-        #high = self.high
         self.control = panel = TraitsUIPanel( parent, -1 )
         sizer  = wx.BoxSizer( wx.HORIZONTAL )
         fvalue = self.value
 
-        try:
-            fvalue_text = self.format % fvalue
-            1 / (self.low <= fvalue <= self.high)
-        except:
+        if not (self.low <= fvalue <= self.high):
             fvalue_text = ''
-            fvalue      = low
+            fvalue = self.low
+        else:
+            try:
+                fvalue_text = self.format % fvalue
+            except (ValueError, TypeError), e:
+                fvalue_text = ''
 
         ivalue = self._convert_to_slider(fvalue)
 
@@ -191,12 +213,21 @@ class SimpleSliderEditor ( Editor ):
     def update_object_on_enter ( self, event ):
         """ Handles the user pressing the Enter key in the text field.
         """
+
+        # There are cases where this method is called with self.control == None.
+        if self.control is None:
+            return
+
         try:
             try:
-                value = self.evaluate( self.control.text.GetValue().strip() )
+                value = self.control.text.GetValue().strip()
+                if self.factory.is_float:
+                    value = float(value)
+                else:
+                    value = int(value)
             except Exception, ex:
-                # The entered something that didn't eval as a number, (e.g., 'foo')
-                # pretend it didn't happen (i.e. do not change self.value)
+                # The user entered something that didn't eval as a number (e.g., 'foo').
+                # Pretend it didn't happen (i.e. do not change self.value).
                 value = self.value
                 self.control.text.SetValue( str( value ) )
 
@@ -331,7 +362,7 @@ class LogRangeSliderEditor ( SimpleSliderEditor ):
 #  'LargeRangeSliderEditor' class:
 #-------------------------------------------------------------------------------
 
-class LargeRangeSliderEditor ( Editor ):
+class LargeRangeSliderEditor ( BaseRangeEditor ):
     """ A slider editor for large ranges.
 
        The editor displays a slider and a text field. A subset of the total
@@ -355,9 +386,6 @@ class LargeRangeSliderEditor ( Editor ):
     # High end of displayed range
     cur_high = Float
 
-    # Function to evaluate textual user input
-    evaluate = Any
-
     # Flag indicating that the UI is in the process of being updated
     ui_changing = Bool( False )
 
@@ -496,7 +524,19 @@ class LargeRangeSliderEditor ( Editor ):
         """ Handles the user pressing the Enter key in the text field.
         """
         try:
-            self.value = value = self.evaluate( self.control.text.GetValue().strip() )
+            value = self.control.text.GetValue().strip()
+            try:
+                if self.factory.is_float:
+                    value = float(value)
+                else:
+                    value = int(value)
+            except Exception, ex:
+                # The user entered something that didn't eval as a number (e.g., 'foo').
+                # Pretend it didn't happen (i.e. do not change self.value).
+                value = self.value
+                self.control.text.SetValue( str( value ) )
+
+            self.value = value
             self.control.text.SetBackgroundColour(OKColor)
             self.control.text.Refresh()
             # Update the slider range.
@@ -688,7 +728,7 @@ class LargeRangeSliderEditor ( Editor ):
 #  'SimpleSpinEditor' class:
 #-------------------------------------------------------------------------------
 
-class SimpleSpinEditor ( Editor ):
+class SimpleSpinEditor ( BaseRangeEditor ):
     """ A simple style of range editor that displays a spin box control.
     """
 
@@ -791,6 +831,13 @@ class RangeTextEditor ( TextEditor ):
         value that is outside the allowed range, the background of the field
         changes color to indicate an error.
     """
+
+    #---------------------------------------------------------------------------
+    #  Trait definitions:
+    #---------------------------------------------------------------------------
+
+    # Function to evaluate floats/ints
+    evaluate = Any
     
     #---------------------------------------------------------------------------
     #  Finishes initializing the editor by creating the underlying toolkit
@@ -813,6 +860,9 @@ class RangeTextEditor ( TextEditor ):
         
         if self.factory.auto_set:
             wx.EVT_TEXT( parent, control.GetId(), self.update_object )
+
+        self.evaluate = self.factory.evaluate
+        self.sync_value( self.factory.evaluate_name, 'evaluate', 'from' )
            
         self.control = control
         self.set_tooltip()
@@ -824,8 +874,31 @@ class RangeTextEditor ( TextEditor ):
     def update_object ( self, event ):
         """ Handles the user entering input data in the edit control.
         """
+
+        # There are cases where this method is called with self.control == None.
+        if self.control is None:
+            return
+
+        value = self.control.GetValue()
+
+        # Try to convert the string value entered by the user to a numerical value.
         try:
-            self.value = eval(self.control.GetValue())
+            if self.evaluate is not None:
+                value = self.evaluate(value)
+            else:
+                if self.factory.is_float:
+                    value = float(value)
+                else:
+                    value = int(value)
+        except Exception, excp:
+            # The conversion failed.
+            self.error(excp)
+            return
+
+        # Try to assign the numerical value to the trait.
+        # This may fail because of constraints on the trait.
+        try:
+            self.value = value
             self.control.SetBackgroundColour(OKColor)
             self.control.Refresh()
             if self._error is not None:
