@@ -36,11 +36,14 @@ from enthought.traits.api \
 from enthought.traits.ui.api \
     import View, Item, Editor
 
+from enthought.traits.ui.editors.table_editor \
+    import ReversedList
+
 from enthought.traits.ui.table_filter \
     import TableFilter
 
 from enthought.traits.ui.ui_traits \
-    import ATheme, SequenceTypes
+    import SequenceTypes
 
 from enthought.pyface.ui.wx.grid.api \
     import GridModel, GridSortEvent
@@ -92,15 +95,6 @@ class TableModel ( GridModel ):
     # Display the table items in reverse order?
     reverse = Bool( False )
     
-    # The theme to use for normal cells:
-    cell_theme = ATheme
-    
-    # The theme to use for alternate row cells (defaults to 'cell_theme):
-    alt_theme = ATheme
-    
-    # The theme to use for selected cells:
-    selected_theme = ATheme
-
     # Event fired when the table has been sorted
     sorted = Event
 
@@ -503,10 +497,10 @@ class TableModel ( GridModel ):
         for selection in selection_list:
             row = -1
             if selection.obj is not None:
-               try:
-                   row = items.index( selection.obj )
-               except ValueError:
-                   continue
+                try:
+                    row = items.index( selection.obj )
+                except ValueError:
+                    continue
 
             column = -1
             if selection.name != '':
@@ -589,8 +583,9 @@ class TableModel ( GridModel ):
         """ Returns a wxColour object specifying the text color of the
             specified cell.
         """
-        return self.__get_column( col ).get_text_color(
-                   self.get_filtered_item( row ) )
+        column = self.__get_column(col)
+        item = self.get_filtered_item(row)
+        return column.get_text_color(item)
 
     def get_cell_font ( self, row, col ):
         """ Returns a wxFont object specifying the font of the specified cell.
@@ -865,94 +860,3 @@ class TableModel ( GridModel ):
         for i, col in enumerate( self.__get_columns() ):
             if name == col.name:
                 return i
-
-#-------------------------------------------------------------------------------
-#  'ReversedList' class:
-#-------------------------------------------------------------------------------
-
-class ReversedList ( object ):
-    """ A list whose order is the reverse of its input.
-    """
-    #---------------------------------------------------------------------------
-    #  Initializes the object:
-    #---------------------------------------------------------------------------
-
-    def __init__ ( self, list ):
-        self.list = list
-
-    #---------------------------------------------------------------------------
-    #  Inserts a value at a specified index in the list:
-    #---------------------------------------------------------------------------
-
-    def insert ( self, index, value ):
-        """ Inserts a value at a specified index in the list.
-        """
-        return self.list.insert( self._index( index - 1 ), value )
-
-    #---------------------------------------------------------------------------
-    #  Returns the index of the first occurence of the specified value in the
-    #  list:
-    #---------------------------------------------------------------------------
-
-    def index ( self, value ):
-        """ Returns the index of the first occurence of the specified value in
-            the list.
-        """
-        list = self.list[:]
-        list.reverse()
-        
-        return list.index( value )
-
-    #---------------------------------------------------------------------------
-    #  Returns the length of the list:
-    #---------------------------------------------------------------------------
-
-    def __len__ ( self ):
-        """ Returns the length of the list.
-        """
-        return len( self.list )
-
-    #---------------------------------------------------------------------------
-    #  Returns the value at a specified index in the list:
-    #---------------------------------------------------------------------------
-
-    def __getitem__ ( self, index ):
-        """ Returns the value at a specified index in the list.
-        """
-        return self.list[ self._index( index ) ]
-
-    #---------------------------------------------------------------------------
-    #  Sets a slice of a list to the contents of a specified sequence:
-    #---------------------------------------------------------------------------
-
-    def __setslice__ ( self, i, j, values ):
-        """ Sets a slice of a list to the contents of a specified sequence.
-        """
-        return self.list.__setslice__( self._index( i ), self._index( j ),
-                                       values )
-
-    #---------------------------------------------------------------------------
-    #  Deletes the item at a specified index:
-    #---------------------------------------------------------------------------
-
-    def __delitem__ ( self, index ):
-        """ Deletes the item at a specified index.
-        """
-        return self.list.__delitem__( self._index( index ) )
-
-    #---------------------------------------------------------------------------
-    #  Returns the 'reversed' value for a specified index:
-    #---------------------------------------------------------------------------
-
-    def _index ( self, index ):
-        """ Returns the "reversed" value for a specified index.
-        """
-        if index < 0:
-            return (-1 - index)
-            
-        result = (len( self.list ) - index - 1)
-        if result >= 0:
-            return result
-            
-        return index
-
