@@ -101,7 +101,7 @@ class wxListCtrl ( wx.ListCtrl, TextEditMixin ):
         # if the selected is editable, then we have to init the mixin
         if can_edit:
             TextEditMixin.__init__(self, edit_labels)
-
+            
     def SetVirtualData(self, row, col, text):
         # this method is called but the job is already done by
         # the _end_label_edit method. Commmented code is availabed
@@ -165,6 +165,7 @@ class wxListCtrl ( wx.ListCtrl, TextEditMixin ):
         editor = self._editor
         return editor.adapter.get_text( editor.object, editor.name,
                                         row, column )
+        
 #-------------------------------------------------------------------------------
 #  'TabularEditor' class:
 #-------------------------------------------------------------------------------
@@ -228,6 +229,9 @@ class TabularEditor ( Editor ):
 
     # An image being converted:
     image = Image
+    
+    # Flag for marking whether the update was within the visible area
+    _update_visible = Bool(False)
 
     #---------------------------------------------------------------------------
     #  Finishes initializing the editor by creating the underlying toolkit
@@ -266,7 +270,7 @@ class TabularEditor ( Editor ):
                                              can_edit = factory.editable,
                                              edit_labels = factory.editable_labels)
         control._editor = self
-
+        
         # Create the list control column:
         #fixme: what do we do here?
         #control.InsertColumn( 0, '' )
@@ -422,6 +426,23 @@ class TabularEditor ( Editor ):
         self.control.RefreshRect(
              self.control.GetItemRect( row, wx.LIST_RECT_BOUNDS ) )
 
+    def _update_editor ( self, object, name, old_value, new_value ):
+        """ Performs updates when the object trait changes.
+            Overloads enthought.traits.ui.editor.UIEditor
+        """
+        control = self.control
+        n       = self.adapter.len( self.object, self.name )
+        top     = control.GetTopItem()
+        pn      = control.GetCountPerPage()
+        bottom = min(top + pn - 1, n, len(old_value))
+        
+        if old_value[top:bottom] != new_value[top:bottom]:
+            self._update_visible = True
+            
+        super(TabularEditor, self)._update_editor(object, name, 
+                                                  old_value, new_value)
+        
+        
     def update_editor ( self ):
         """ Updates the editor when the object trait changes externally to the
             editor.
@@ -430,12 +451,14 @@ class TabularEditor ( Editor ):
         n       = self.adapter.len( self.object, self.name )
         top     = control.GetTopItem()
         pn      = control.GetCountPerPage()
+        bottom = min(top + pn - 1, n)
 
-        control.DeleteAllItems()
         control.SetItemCount( n )
-        if n > 0:
-            control.RefreshItems( 0, n - 1 )
-
+                
+        if self._update_visible:
+            control.RefreshItems( 0, n-1 )
+            self._update_visible = False
+                
         if len( self.multi_selected_rows ) > 0:
             self._multi_selected_rows_changed( self.multi_selected_rows )
         if len( self.multi_selected ) > 0:
@@ -443,7 +466,7 @@ class TabularEditor ( Editor ):
 
         edit, self.edit = self.edit, False
         row,  self.row  = self.row,  None
-
+        
         if row is not None:
             if row >= n:
                 row -= 1
@@ -451,10 +474,11 @@ class TabularEditor ( Editor ):
                     row = None
 
         if row is None:
-            visible = top + pn - 2
+            visible = bottom
             if visible >= 0 and visible < control.GetItemCount():
                 control.EnsureVisible( visible )
             return
+        
 
         if 0 <= (row - top) < pn:
             control.EnsureVisible( top + pn - 2 )
@@ -1000,7 +1024,7 @@ class TabularEditor ( Editor ):
             adapter   = self.adapter
             self.row  = self.control.GetItemCount()
             self.edit = True
-            adapter.insert( self.object, self.name, self.row,
+            adapter.insert( self.object, self.name, row,
                            adapter.get_default_value( self.object, self.name ) )
 
     def _insert_current ( self ):
