@@ -137,7 +137,7 @@ class SimpleEditor ( Editor ):
     
     # The event fired when the application wants to veto an operation:
     veto = Event
-    
+
     #-- Private Traits ---------------------------------------------------------
     
     # An icon used by a TreeNode:
@@ -276,6 +276,9 @@ class SimpleEditor ( Editor ):
         wx.EVT_TREE_END_LABEL_EDIT(   tree, tid, self._on_tree_end_label_edit )
         wx.EVT_TREE_ITEM_GETTOOLTIP(  tree, tid, self._on_tree_item_gettooltip )
 
+        # Set up general mouse events
+        wx.EVT_MOTION(tree, self._on_hover)
+        
         # Synchronize external object traits with the editor:
         self.sync_value( factory.selected, 'selected' )
         self.sync_value( factory.click,    'click',  'to' )
@@ -364,6 +367,10 @@ class SimpleEditor ( Editor ):
         if factory.hide_root:
             style |= (wx.TR_HIDE_ROOT | wx.TR_LINES_AT_ROOT)
 
+        if factory.selection_mode != 'single':
+            style |= wx.TR_MULTIPLE | wx.TR_EXTENDED
+
+
         return style
 
     #---------------------------------------------------------------------------
@@ -676,40 +683,6 @@ class SimpleEditor ( Editor ):
             
         return copy.deepcopy( new_object )
                 
-    #---------------------------------------------------------------------------
-    #  Returns whether a specified object has any children:
-    #---------------------------------------------------------------------------
-    
-    def _has_children ( self, node, object ):
-        """ Returns whether a specified object has any children.
-        """
-        return (node.allows_children( object ) and node.has_children( object )) 
-                
-    #---------------------------------------------------------------------------
-    #  Returns whether a given object is droppable on the node:    
-    #---------------------------------------------------------------------------
-                                
-    def _is_droppable ( self, node, object, add_object, for_insert ):
-        """ Returns whether a given object is droppable on the node.
-        """
-        if for_insert and (not node.can_insert( object )):
-            return False
-            
-        return node.can_add( object, add_object )
-                           
-    #---------------------------------------------------------------------------
-    #  Returns a droppable version of a specified object:    
-    #---------------------------------------------------------------------------
-                                                      
-    def _drop_object ( self, node, object, dropped_object, make_copy = True ):
-        """ Returns a droppable version of a specified object.
-        """
-        new_object = node.drop_object( object, dropped_object )
-        if (new_object is not dropped_object) or (not make_copy):
-            return new_object
-            
-        return copy.deepcopy( new_object )
-
     #---------------------------------------------------------------------------
     #  Returns the icon index for the specified object:
     #---------------------------------------------------------------------------
@@ -1192,18 +1165,31 @@ class SimpleEditor ( Editor ):
         # Get the new selection:
         object      = None
         not_handled = True
-        nid         = self._tree.GetSelection()
-        if nid.IsOk():
-            # If there is a real selection, get the associated object:
-            expanded, node, object = self._get_node_data( nid )
+        nids        = self._tree.GetSelections()
 
-            # Try to inform the node specific handler of the selection:
-            not_handled = node.select( object )
+	selected = []
+	for nid in nids:
+            if nid.IsOk():
+                # If there is a real selection, get the associated object:
+                expanded, node, sel_object = self._get_node_data( nid )
+		selected.append(sel_object)
+
+                # Try to inform the node specific handler of the selection,
+                # if there are multiple selections, we only care about the
+                # first (or maybe the last makes more sense?)
+		if nid == nids[0]:
+		    object = sel_object
+                    not_handled = node.select( object )
 
         # Set the value of the new selection:
-        self._no_update_selected = True
-        self.selected            = object
-        self._no_update_selected = False
+	if self.factory.selection_mode == 'single':
+            self._no_update_selected = True
+            self.selected = object
+            self._no_update_selected = False
+	else:
+            self._no_update_selected = True
+	    self.selected = selected
+            self._no_update_selected = False
 
         # If no one has been notified of the selection yet, inform the editor's
         # select handler (if any) of the new selection:
@@ -1257,6 +1243,22 @@ class SimpleEditor ( Editor ):
             # Allow the editor view to show any changes that have occurred:
             editor.Thaw()
 
+    def _on_hover(self, event):
+        """ Handles the mouse moving over a tree node
+        """
+        id, flags = self._tree.HitTest(event.GetPosition())
+        if flags & wx.TREE_HITTEST_ONITEMLABEL:
+            expanded, node, object = self._get_node_data( id )
+            if self.factory.on_hover is not None:
+                self.ui.evaluate( self.factory.on_hover, object )
+                self._veto = True
+        elif self.factory.on_hover is not None:
+            self.ui.evaluate( self.factory.on_hover, None )
+	# allow other events to be processed
+	event.Skip(True)
+            
+                
+
     #---------------------------------------------------------------------------
     #  Handles a tree item being activated (i.e. double clicked):
     #---------------------------------------------------------------------------
@@ -1513,8 +1515,10 @@ class SimpleEditor ( Editor ):
         parent = self._menu_parent_node
         if isinstance( parent, ObjectTreeNode ):
             can_rename = parent.can_rename( self._menu_parent_object )
+        elif parent is not None:
+            can_rename = parent.can_rename( object )
         else:
-            can_rename = ((parent is not None) and parent.can_rename( object ))
+            can_rename = True
         return (can_rename and self._menu_node.can_rename_me( object ))
 
 #----- Drag and drop event handlers: -------------------------------------------
@@ -1990,4 +1994,4 @@ class ObjectLabel ( HasStrictTraits ):
                         kind    = 'modal',
                         buttons = [ 'OK', 'Cancel' ] )
 
-### EOF #######################################################################
\ No newline at end of file
+### EOF #######################################################################
